"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LodContentExporterImpl = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs"));
const path = tslib_1.__importStar(require("path"));
const inversify_1 = require("inversify");
const types_1 = require("@services/types");
const constants_1 = require("@model/constants");
let LodContentExporterImpl = class LodContentExporterImpl {
    keyGenerator;
    baseLodWordExporter;
    nounExporter;
    verbExporter;
    labelProvider;
    constructor(keyGenerator, baseLodWordExporter, nounExporter, verbExporter, labelProvider) {
        this.keyGenerator = keyGenerator;
        this.baseLodWordExporter = baseLodWordExporter;
        this.nounExporter = nounExporter;
        this.verbExporter = verbExporter;
        this.labelProvider = labelProvider;
    }
    export(apkg, language, flashcard, types, outputDirectory) {
        const wordsFolder = path.join(outputDirectory, constants_1.CONSTANTS.WORDS_FOLDER);
        const lodAudiosFolder = path.join(outputDirectory, constants_1.CONSTANTS.AUDIOS_FOLDER);
        const wordId = this.keyGenerator.generate(flashcard);
        const wordFile = path.join(wordsFolder, `${wordId}.json`);
        if (!fs.existsSync(wordFile)) {
            console.error(`${flashcard} not found!`);
            return;
        }
        const word = JSON.parse(fs.readFileSync(wordFile).toString());
        let flashcardBack = "<div style=\"text-align: left\">";
        let shouldAddCard = false;
        for (const type of word.types) {
            if (types.length !== 0 && !types.includes(type.type)) {
                continue;
            }
            shouldAddCard = true;
            flashcardBack += "<div style=\"margin-bottom: 25px;\">";
            if (!!type.details.audio) {
                apkg.addMedia(`${type.details.audio}`, fs.readFileSync(path.join(lodAudiosFolder, `${type.details.audio}`)));
            }
            switch (type.type) {
                case "noun":
                    flashcardBack += this.nounExporter.rerieveWordTypeHeader(language, word, type);
                    break;
                case "verb":
                    flashcardBack += this.verbExporter.rerieveWordTypeHeader(language, word, type);
                    break;
                case "preposition":
                case "conjunction":
                case "adjective":
                case "adverb":
                    flashcardBack += this.baseLodWordExporter.rerieveWordTypeHeader(language, word, type);
                    break;
            }
            flashcardBack += this.retrieveTranslationContent(language, type);
            flashcardBack += "</div>";
        }
        flashcardBack += "</div>";
        if (shouldAddCard) {
            apkg.addCard(`<div style="text-align: left">${word.word}</div>`, flashcardBack);
        }
    }
    retrieveTranslationContent(language, type) {
        if (type.meanings.length === 0) {
            return "";
        }
        let content = `<ol>`;
        for (const meaning of type.meanings) {
            const translation = meaning.translations.find(it => it.language === language);
            if (!!translation?.translation) {
                let polyLex = "";
                if (!!meaning.polyLex) {
                    polyLex = `<span style="color: #b00c12;">${meaning.polyLex}</span><br>`;
                }
                let complement = "";
                if (!!translation.complement) {
                    complement = ` [${translation.complement}]`;
                }
                let examples = "";
                if (!!meaning.examples) {
                    examples = `<br><span style="font-size: 85%; color: #595959;">${this.labelProvider.get("EXAMPLE", language)}</span><br>${meaning.examples.map(it => this.buildExample(it, language)).join("<br>")}`;
                }
                content += `<li>${polyLex}${translation.translation}${complement}${examples}</li>`;
            }
        }
        content += `</ol>`;
        return content;
    }
    buildExample(example, language) {
        let usages = example.usage.split(" ")
            .filter(it => it !== "ALLG")
            .map(it => `<span style="color: #595959; padding-left: 2px; padding-right: 2px; border-style: solid; border-color: #A4A4A4; border-width: thin; font-size: 90%;">${this.labelProvider.get(it + "_USAGE", language)}</span> `)
            .join(" ");
        return `${usages || ""}${example.example}`;
    }
};
LodContentExporterImpl = tslib_1.__decorate([
    (0, inversify_1.injectable)(),
    tslib_1.__param(0, (0, inversify_1.inject)(types_1.TYPES.WordIdGenerator)),
    tslib_1.__param(1, (0, inversify_1.inject)(types_1.TYPES.BaseLodWordExporter)),
    tslib_1.__param(2, (0, inversify_1.inject)(types_1.TYPES.NounExporter)),
    tslib_1.__param(3, (0, inversify_1.inject)(types_1.TYPES.VerbExporter)),
    tslib_1.__param(4, (0, inversify_1.inject)(types_1.TYPES.LabelProvider)),
    tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object])
], LodContentExporterImpl);
exports.LodContentExporterImpl = LodContentExporterImpl;
