"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseLodWordExtractorImpl = void 0;
const tslib_1 = require("tslib");
const types_1 = require("@services/types");
const inversify_1 = require("inversify");
let BaseLodWordExtractorImpl = class BaseLodWordExtractorImpl {
    wordIdGenerator;
    constructor(wordIdGenerator) {
        this.wordIdGenerator = wordIdGenerator;
    }
    extract(lodKey, lodWordType, word, structure) {
        const wordType = this.retrieveWordType(lodWordType);
        return {
            id: this.wordIdGenerator.generate(word),
            word: word,
            types: [{
                    type: wordType,
                    lodKey: lodKey,
                    details: this.extractDetails(lodWordType, structure),
                    meanings: this.extractMeanings(lodWordType, word, structure)
                }]
        };
    }
    retrieveWordType(lodWordType) {
        switch (lodWordType) {
            case "SUBST":
                return "noun";
            case "ADJ":
                return "adjective";
            case "ADV":
                return "adverb";
            case "PREP":
                return "preposition";
            case "VRB":
                return "verb";
            case "CONJ":
                return "conjunction";
            default: throw new Error(`${lodWordType} not recognized.`);
        }
    }
    extractDetails(lodWordType, structure) {
        const { variationOfLodKey, variationType } = this.extractVariantOf(lodWordType, structure);
        return {
            variationOfLodKey: variationOfLodKey,
            variationType: variationType
        };
    }
    extractVariantOf(lodWordType, structure) {
        let variationOfLodKey = undefined;
        let variationType = undefined;
        const variantOfStructure = structure[`lod:RENVOI-${lodWordType}`];
        if (!!variantOfStructure) {
            variationOfLodKey = variantOfStructure[0]["attributes"]["lod:REF-ID-ITEM-ADRESSE"].slice(0, -3);
            variationType = Object.keys(variantOfStructure[0])[2];
            switch (variationType) {
                case "lod:VARIANTE-HOMOSEME":
                case "lod:VARIANTE-ORTHOGRAPHIQUE":
                case "lod:VARIANTE-LOCALE":
                    variationType = "VARIANT_OF";
                    break;
                case "lod:FORME-FEM":
                    variationType = "FEMININE_FORM_OF";
                    break;
                case "lod:FORME-ABREGEE":
                    variationType = "SHORT_FORM_OF";
                    break;
                case "lod:FORME-DIMINUTIVE":
                    variationType = "DIMINUTIVE_FORM_OF";
                    break;
                case "lod:FORME-MASC":
                    variationType = "MASCULINE_FORM_OF";
                    break;
                default: throw new Error(`${variationType} not recognized.`);
            }
        }
        return { variationOfLodKey, variationType };
    }
    extractMeanings(lodWordType, word, structure) {
        let meanings = [];
        const meaningsStructure = structure[`lod:TRAITEMENT-LING-${lodWordType}`];
        if (!!meaningsStructure) {
            for (const meaningStructure of meaningsStructure) {
                meanings = meanings.concat(this.extractMeaningsFromStructure(word, meaningStructure));
            }
        }
        return meanings;
    }
    extractMeaningsFromStructure(word, meaningStructure) {
        const meanings = [];
        const translationUnitStructures = meaningStructure["lod:UNITE-TRAD"];
        for (const translationUnitStructure of translationUnitStructures) {
            const translationStructures = translationUnitStructure["lod:PAS-DE-TRAD-SUBORDONNANTE"];
            for (const translationStructure of translationStructures) {
                const translation = translationStructure["lod:UNITE-DE-SENS"][0];
                meanings.push({
                    examples: this.extractExamples(word, translation),
                    synonyms: this.extractSynonyms(translation),
                    polyLex: this.extractPolyLex(translation),
                    translations: [
                        this.extractTranslation("ALL", translation),
                        this.extractTranslation("FR", translation),
                        this.extractTranslation("PO", translation),
                        this.extractTranslation("EN", translation)
                    ]
                });
            }
        }
        return meanings;
    }
    extractExamples(word, translationStructure) {
        const examples = [];
        const examplesStructure = translationStructure["lod:EXEMPLIFICATION"][0]["lod:EXEMPLE"];
        for (const exampleStructure of examplesStructure) {
            const usage = exampleStructure["attributes"]["lod:MARQUE-USAGE"].trim();
            examples.push({
                example: exampleStructure["lod:TEXTE-EX"][0]["$$"].map((it) => {
                    if (it["#name"] === "lod:ABREV-AD" && it["_"].includes("."))
                        return word + it["_"].split(".")[1];
                    return it["_"];
                }).join(" "),
                usage: usage
            });
        }
        return examples;
    }
    extractSynonyms(translationStructure) {
        const synonymsStructure = translationStructure["lod:SYNONYMES"][0]["lod:SYN-PRESENTS"];
        if (!synonymsStructure) {
            return [];
        }
        const synonyms = [];
        for (const synonymStructure of synonymsStructure) {
            synonyms.push(synonymStructure["lod:TERME-SYN"][0]);
        }
        return [];
    }
    extractPolyLex(translationStructure) {
        const polyLexStructure = translationStructure["lod:UNITE-POLYLEX-LUX"];
        if (!polyLexStructure) {
            return undefined;
        }
        return polyLexStructure[0];
    }
    extractTranslation(languageKey, translationStructure) {
        const translationObj = translationStructure[`lod:EQUIV-TRAD-${languageKey}`];
        if (!translationObj) {
            return {
                language: languageKey,
                translation: undefined
            };
        }
        const translation = translationObj[0][`lod:ET${languageKey[0]}-EXPLICITE`][0]["_"];
        let complement = undefined;
        let complementStructure = translationObj[0][`lod:RS-ET${languageKey[0]}-PRESENTE`];
        if (!!complementStructure) {
            complement = complementStructure[0].trim();
        }
        return {
            language: languageKey,
            translation: translation,
            complement: complement
        };
    }
};
BaseLodWordExtractorImpl = tslib_1.__decorate([
    (0, inversify_1.injectable)(),
    tslib_1.__param(0, (0, inversify_1.inject)(types_1.TYPES.WordIdGenerator)),
    tslib_1.__metadata("design:paramtypes", [Object])
], BaseLodWordExtractorImpl);
exports.BaseLodWordExtractorImpl = BaseLodWordExtractorImpl;
